    /**
     * File: JLists.java <BR>
     * A simple forward-directed, singly linked list of Nodes; <BR>
     * each Node contains a single int datum. <BR>
     * Node is an inner class.
     * 
     * @author Bary W Pollack
     * @version Nov. 17, 2000
     * @see Node
     */

class List {

    /**
     * Predicate.  True if the list structure is empty, false otherwise.
     */

    public boolean isEmpty() { return (head == null); }

    /**
     * Creates a new Node containing the datum, nValue; 
     * adds the new node at the head of the list.
     * 
     * @param nValue The int value to be added to the list.
     */

    public void add(int nValue) { head = new Node(nValue, head); }

    /**
     * Creates a new node with datum, nValue; inserts the new node at the proper
     * location in the list so that the list remains sorted in ascending order.
     * 
     * @param nValue The int value to be inserted in to the list.
     */

    public void insert(int nValue) {
        if (head == null || nValue < head.nValue)
            head = new Node(nValue, head);
        else {
            Node p = head, q = head.next;
            while (q != null && nValue > q.nValue) {
                p = q; q = q.next;
            }
            p.next = new Node(nValue, q);
        }
    }

    /**
     * Removes the head node of the list; returns its datum as the value.
     */

    public int remove() {
        int nValue = 0;
        if (head != null) {
            nValue = head.nValue;
            head = head.next;
        }
        return nValue;
    }

    /**
     * Returns a copy of the datum of the node at the head of the list.
     */

    public int peek() {
        return (head != null) ? head.nValue : 0;
    }

    /**
     * Reverses the list.
     */

    public void reverse() {
        List revList = new List();
        while (head != null) {
            revList.add(head.nValue);
            head = head.next;
        }
        head = revList.head;
    }

    /**
     * Merges two sorted lists together, returning the sorted merged result.
     */

    public void merge(List list) {
        List mergedList = new List();
        while (head != null || list.head != null) {
            if (head == null) {
                mergedList.add(list.head.nValue);
                list.head = list.head.next;
            } else if (list.head == null || head.nValue < list.head.nValue) {
                mergedList.add(head.nValue);
                head = head.next;
            } else {
                mergedList.add(list.head.nValue);
                list.head = list.head.next;
            }
        }
        mergedList.reverse();
        head = mergedList.head;
    }

    /**
     * Sort the current list into ascending order 
     * using the insertion sort algorithm.
     */
    
    public void insertionSort() {
        List list = new List();
        while (head != null)
            list.insert(remove());
        head = list.head;
    }

    /**
     * Calculate the length of the list.
     * 
     * @return The list's length.
     */

    public int length() {
        int nLength = 0;
        for (Node n = head; n != null; n = n.next)
            ++nLength;
        return nLength;
    }

    /**
     * Convert tbe current string data into a string.
     */

    public String toString() {
        String s = "(";
        for (Node n = head; n != null; n = n.next)
            s = s + " " + n.nValue;
        return s + " )";
    }

    private Node head;

    /**
     * A single node of the list structure. <BR>
     * A Node is implemented as an inner class. <BR>
     * It contains: <BR>
     *      nValue - an int, <BR>
     *      next - a link to the next node in the list.
     */

    private class Node {
        Node() { this(0, null); }
        Node(int nValue) { this(nValue, null); }
        Node(int nValue, Node next) { this.nValue = nValue; this.next = next; }

        private int   nValue;
        private Node  next;
    }
}

/**
 * Driver to demonstrate use of the List class.
 * 
 * @author Bary W Pollack
 * @version Nov. 17, 2000
 * @see List
 * @see Node
 */

public class JLists {

    /**
     * Create several lists of varying lengths. <BR>
     * Demonstrate that insertionSort() and merge work properly.
     */

    JLists() {
        List mergedList = new List();
        for (int i = 0; i < 13; i = (i < 3) ? i + 1 : i + 3) {
            List list = createList(i);
            System.out.println("Unsorted: " + list.length() + ": " + list);
            list.insertionSort();
            System.out.println("Sorted:   " + list.length() + ": " + list);
            mergedList.merge(list);
            System.out.println("Merged:   " + mergedList.length() + ": " + mergedList);
            System.out.println();
        }
    }

    /**
     * Create a list containing n random int's in the range 0..99
     * 
     * @param n The number of items in the list.
     * @return The created list containing n random int's.
     */

    List createList(int n) {
        List list = new List();
        for (int i = 0; i < n; i++)
            list.add((int) Math.rint(100.0 * Math.random()));
        return list;
    }

    public static void main(String[] args) {
        System.out.println("Demonstrate JList - add, insert, insertionSort, merge...");
        System.out.println();
        new JLists();
        System.out.println("*** Fin ***");
        System.out.println();
    }

}

/* ..... Output from the program .....

Demonstrate JList - add, insert, insertionSort, merge...

Unsorted: 0: ( )
Sorted:   0: ( )
Merged:   0: ( )

Unsorted: 1: ( 79 )
Sorted:   1: ( 79 )
Merged:   1: ( 79 )

Unsorted: 2: ( 26 57 )
Sorted:   2: ( 26 57 )
Merged:   3: ( 26 57 79 )

Unsorted: 3: ( 41 6 56 )
Sorted:   3: ( 6 41 56 )
Merged:   6: ( 6 26 41 56 57 79 )

Unsorted: 6: ( 17 30 21 82 82 9 )
Sorted:   6: ( 9 17 21 30 82 82 )
Merged:   12: ( 6 9 17 21 26 30 41 56 57 79 82 82 )

Unsorted: 9: ( 73 14 37 27 74 27 72 2 75 )
Sorted:   9: ( 2 14 27 27 37 72 73 74 75 )
Merged:   21: ( 2 6 9 14 17 21 26 27 27 30 37 41 56 57 72 73 74 75 79 82 82 )

Unsorted: 12: ( 41 95 20 9 63 43 49 14 50 89 41 83 )
Sorted:   12: ( 9 14 20 41 41 43 49 50 63 83 89 95 )
Merged:   33: ( 2 6 9 9 14 14 17 20 21 26 27 27 30 37 41 41 41 43 49 50 56 57 63 72 73 74 75 79 82 82 83 89 95 )

*** Fin ***

..... end of output ..... */
